
{*****************************************************************************
 * Name           : KNGDateUtil                                              *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 09/06/1999                                               *
 * Description    : This unit will contain several Date / Time functions that*
 *                  I created at some point.  These functions and procedures *
 *                  were not available in Delphi.  This unit requires the    *
 *                  DateUtil unit from the RxLib since it uses some of the   *
 *                  functions and procedures defined in that unit.           *
 *                  Examine the Procedure and Function Header comments to see*
 *                  what the Functions / Procedure actually do.              *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 09/06/1999 Lesage Stefaan  Initial creation of the Unit                   *
 *****************************************************************************}

unit KNGDateUtil;

{$I KNG.INC}

interface

{$IFDEF RXLIB}
function FirstDayOfThisWeek (ADate: TDateTime; AFirstDayOfWeek : Integer): TDateTime;
function LastDayOfThisWeek (ADate: TDateTime; AFirstDayOfWeek : Integer): TDateTime;
function FirstDayOfNextWeek (ADate: TDateTime; AFirstDayOfWeek : Integer): TDateTime;
function LastDayOfNextWeek (ADate: TDateTime; AFirstDayOfWeek : Integer): TDateTime;
function FirstDayOfPriorWeek (ADate: TDateTime; AFirstDayOfWeek : Integer): TDateTime;
function LastDayOfPriorWeek (ADate: TDateTime; AFirstDayOfWeek : Integer): TDateTime;
{$ENDIF}

implementation

uses
  SysUtils, {$IFDEF RXLIB} DateUtil, {$ENDIF} Dialogs, KNGConst;

{$IFDEF RXLIB}
{*****************************************************************************
 * Name           : FirstDayOfThisWeek                                       *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 09/06/1999                                               *
 * Arguments      :                                                          *
 *                                                                           *
 *  Kind      Name             Type          Description                     *
 *  ----      ----             --------      -----------                     *
 *            ADate            TDateTime     The date from which the calcu-  *
 *                                           lation will start.              *
 *            AFirstDayOfWeek  Integer       This is the day on which a new  *
 *                                           week starts.                    *
 *                                           1 .. 7 for Sunday .. Saturday   *
 *                                                                           *
 * Return Values  : TDateTime                                                *
 * Description    : Returns the first day of the week where the given date   *
 *                  falls in.  This all depends on the AFirstDayOfWeek which *
 *                  is the day on which a new week begins                    *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 09/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function FirstDayOfThisWeek (ADate: TDateTime; AFirstDayOfWeek: Integer):TDateTime;
var
  tmpDate : TDateTime;
begin
  { Check if the AFirstDayOfWeek parameter is a valid number between 1 and 7
    if that is not the case, raise an error, else continue }
  if (AFirstDayOfWeek < 0) or (AFirstDayOfWeek > 7) then
    raise ERangeError.Create(sErrWeekdayOutOfRange)
  else
  begin
    tmpDate := ADate;
    while DayOfWeek(tmpDate) <> AFirstDayOfWeek do
    begin
      tmpDate := IncDay(tmpDate, -1);
    end;
    Result := tmpDate;
  end;
end;

{*****************************************************************************
 * Name           : FirstDayOfNextWeek                                       *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 09/06/1999                                               *
 * Arguments      :                                                          *
 *                                                                           *
 *  Kind      Name             Type          Description                     *
 *  ----      ----             --------      -----------                     *
 *            ADate            TDateTime     The date from which the calcu-  *
 *                                           lation will start.              *
 *            AFirstDayOfWeek  Integer       This is the day on which a new  *
 *                                           week starts.                    *
 *                                           1 .. 7 for Sunday .. Saturday   *
 *                                                                           *
 * Return Values  : TDateTime                                                *
 * Description    : Returns the first day of the week folowing the one where *
 *                  the given date falls in.  This all depends on the        *
 *                  AFirstDayOfWeek which is the day on which a new week     *
 *                  begins.                                                  *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 09/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function FirstDayOfNextWeek (ADate: TDateTime; AFirstDayOfWeek: Integer):TDateTime;
begin
  { Check if the AFirstDayOfWeek parameter is a valid number between 1 and 7
    if that is not the case, raise an error, else continue }
  if (AFirstDayOfWeek < 0) or (AFirstDayOfWeek > 7) then
    raise ERangeError.Create(sErrWeekdayOutOfRange)
  else
    { Get the First day of the week the date falls in and add 7 to the
      result to get the First day of the next week }
    Result := FirstDayOfThisWeek(ADate, AFirstDayOfWeek) + 7;
end;

{*****************************************************************************
 * Name           : FirstDayOfPriorWeek                                      *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 09/06/1999                                               *
 * Arguments      :                                                          *
 *                                                                           *
 *  Kind      Name             Type          Description                     *
 *  ----      ----             --------      -----------                     *
 *            ADate            TDateTime     The date from which the calcu-  *
 *                                           lation will start.              *
 *            AFirstDayOfWeek  Integer       This is the day on which a new  *
 *                                           week starts.                    *
 *                                           1 .. 7 for Sunday .. Saturday   *
 *                                                                           *
 * Return Values  : TDateTime                                                *
 * Description    : Returns the first day of the week preceding the one where*
 *                  the given date falls in.  This all depends on the        *
 *                  AFirstDayOfWeek which is the day on which a new week     *
 *                  begins.                                                  *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 09/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function FirstDayOfPriorWeek (ADate: TDateTime; AFirstDayOfWeek: Integer):TDateTime;
begin
  { Check if the AFirstDayOfWeek parameter is a valid number between 1 and 7
    if that is not the case, raise an error, else continue }
  if (AFirstDayOfWeek < 0) or (AFirstDayOfWeek > 7) then
    raise ERangeError.Create(sErrWeekdayOutOfRange)
  else
    { Get the First day of the week the date falls in and substract 7 from the
      result to get the First day of the previous week }
    Result := FirstDayOfThisWeek(ADate, AFirstDayOfWeek) - 7;
end;

{*****************************************************************************
 * Name           : LastDayOfThisWeek *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 09/06/1999                                               *
 * Arguments      :                                                          *
 *                                                                           *
 *  Kind      Name             Type          Description                     *
 *  ----      ----             --------      -----------                     *
 *            ADate            TDateTime     The date from which the calcu-  *
 *                                           lation will start.              *
 *            AFirstDayOfWeek  Integer       This is the day on which a new  *
 *                                           week starts.                    *
 *                                           1 .. 7 for Sunday .. Saturday   *
 *                                                                           *
 * Return Values  : TDateTime                                                *
 * Description    : Returns the last day of the week where the given date    *
 *                  falls in.  This all depends on the AFirstDayOfWeek which *
 *                  is the day on which a new week begins                    *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 09/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function LastDayOfThisWeek (ADate: TDateTime; AFirstDayOfWeek: Integer):TDateTime;
var
  tmpDate : TDateTime;
begin
  { Check if the AFirstDayOfWeek parameter is a valid number between 1 and 7
    if that is not the case, raise an error, else continue }
  if (AFirstDayOfWeek < 0) or (AFirstDayOfWeek > 7) then
    raise ERangeError.Create(sErrWeekdayOutOfRange)
  else
  begin
    tmpDate := ADate;
    if AFirstDayOfWeek = 1 then
    begin
      while DayOfWeek(tmpDate) <> 7 do
      begin
        tmpDate := IncDay(tmpDate, 1)
      end;
    end
    else
    begin
      while DayOfWeek(tmpDate) <> (AFirstDayOfWeek - 1) do
      begin
        tmpDate := IncDay(tmpDate, 1)
      end;
    end;
    Result := tmpDate;
  end;
end;

{*****************************************************************************
 * Name           : LastDayOfNextWeek                                        *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 09/06/1999                                               *
 * Arguments      :                                                          *
 *                                                                           *
 *  Kind      Name             Type          Description                     *
 *  ----      ----             --------      -----------                     *
 *            ADate            TDateTime     The date from which the calcu-  *
 *                                           lation will start.              *
 *            AFirstDayOfWeek  Integer       This is the day on which a new  *
 *                                           week starts.                    *
 *                                           1 .. 7 for Sunday .. Saturday   *
 *                                                                           *
 * Return Values  : TDateTime                                                *
 * Description    : Returns the last day of the week folowing the one where  *
 *                  the given date falls in.  This all depends on the        *
 *                  AFirstDayOfWeek which is the day on which a new week     *
 *                  begins.                                                  *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 09/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function LastDayOfNextWeek (ADate: TDateTime; AFirstDayOfWeek: Integer):TDateTime;
begin
  { Check if the AFirstDayOfWeek parameter is a valid number between 1 and 7
    if that is not the case, raise an error, else continue }
  if (AFirstDayOfWeek < 0) or (AFirstDayOfWeek > 7) then
    raise ERangeError.Create(sErrWeekdayOutOfRange)
  else
    { Get the last day of the week the date falls in and add 7 to the
      result to get the last day of the next week }
    Result := LastDayOfThisWeek(ADate, AFirstDayOfWeek) + 7;
end;

{*****************************************************************************
 * Name           : LastDayOfPriorWeek                                       *
 * Author         : Lesage Stefaan                                           *
 * Last Revision  : 09/06/1999                                               *
 * Arguments      :                                                          *
 *                                                                           *
 *  Kind      Name             Type          Description                     *
 *  ----      ----             --------      -----------                     *
 *            ADate            TDateTime     The date from which the calcu-  *
 *                                           lation will start.              *
 *            AFirstDayOfWeek  Integer       This is the day on which a new  *
 *                                           week starts.                    *
 *                                           1 .. 7 for Sunday .. Saturday   *
 *                                                                           *
 * Return Values  : TDateTime                                                *
 * Description    : Returns the last day of the week preceding the one where *
 *                  the given date falls in.  This all depends on the        *
 *                  AFirstDayOfWeek which is the day on which a new week     *
 *                  begins.                                                  *
 * History        :                                                          *
 *                                                                           *
 * Date       By              Description                                    *
 * ----       --              -----------                                    *
 * 09/06/1999 Lesage Stefaan  Initial creation of the procedure              *
 *****************************************************************************}

function LastDayOfPriorWeek (ADate: TDateTime; AFirstDayOfWeek: Integer):TDateTime;
begin
  { Check if the AFirstDayOfWeek parameter is a valid number between 1 and 7
    if that is not the case, raise an error, else continue }
  if (AFirstDayOfWeek < 0) or (AFirstDayOfWeek > 7) then
    raise ERangeError.Create(sErrWeekdayOutOfRange)
  else
    { Get the last day of the week the date falls in and substrat 7 from the
      result to get the last day of the week before }
    Result := LastDayOfThisWeek(ADate, AFirstDayOfWeek) - 7;
end;
{$ENDIF}

end.
